'use strict';

const fs = require('node:fs');
const path = require('node:path');
const process = require('node:process');
const ci = require('ci-info');
const unplugin = require('unplugin');
const simpleGit = require('simple-git');
const parseGitUrl = require('git-url-parse');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e.default : e; }

const fs__default = /*#__PURE__*/_interopDefaultCompat(fs);
const path__default = /*#__PURE__*/_interopDefaultCompat(path);
const process__default = /*#__PURE__*/_interopDefaultCompat(process);
const ci__default = /*#__PURE__*/_interopDefaultCompat(ci);
const parseGitUrl__default = /*#__PURE__*/_interopDefaultCompat(parseGitUrl);

async function getRepoInfo(root, extra = {}) {
  const git = simpleGit.simpleGit(root);
  if (!await git.checkIsRepo()) {
    return void 0;
  }
  const [branch, currentCommit, commiter, tags, github, result] = await Promise.all([
    getBranch(git),
    getCommit(git),
    getCommitter(git),
    getTags(git),
    getGitHubUrl(git),
    Promise.all(
      Object.entries(extra).map(async ([key, fn]) => {
        return [key, await fn(git)];
      })
    )
  ]);
  return {
    ...branch,
    ...currentCommit,
    ...commiter,
    ...tags,
    ...github,
    ...Object.fromEntries(result)
  };
}
async function getBranch(git) {
  try {
    const branch = (await git.branch([])).current;
    return { branch };
  } catch (error) {
    return { branch: void 0 };
  }
}
async function getCommit(git) {
  try {
    const log = await git.log(["-1"]);
    const sha = log.latest?.hash;
    const commitMessage = log.latest?.message;
    const author = log.latest?.author_name;
    const authorEmail = log.latest?.author_email;
    const authorDate = log.latest?.date;
    return {
      sha,
      abbreviatedSha: sha?.slice(0, 10),
      commitMessage,
      author,
      authorEmail,
      authorDate
    };
  } catch (error) {
    return {
      sha: void 0,
      abbreviatedSha: void 0,
      commitMessage: void 0,
      author: void 0,
      authorEmail: void 0,
      authorDate: void 0
    };
  }
}
async function getCommitter(git) {
  try {
    const [committer, committerEmail, committerDate] = await Promise.all([
      git.show(["-s", "--format=%cn"]),
      git.show(["-s", "--format=%ce"]),
      git.show(["-s", "--format=%cd"])
    ]);
    return {
      committer,
      committerEmail,
      committerDate
    };
  } catch (error) {
    return {
      committer: void 0,
      committerEmail: void 0,
      committerDate: void 0
    };
  }
}
async function getTags(git) {
  try {
    const hash = await git.revparse(["HEAD"]);
    const tags = await git.tags(["--points-at", hash]);
    const all = await git.tags();
    return { tag: tags.all[tags.all.length - 1], tags: tags.all, lastTag: all.latest };
  } catch (error) {
    return { tags: void 0, lastTag: void 0 };
  }
}
async function getGitHubUrl(git) {
  const remotes = await git.getRemotes(true);
  const origin = remotes.find((remote) => remote.name === "origin");
  const url = origin?.refs.fetch;
  if (url) {
    const parsed = parseGitUrl__default(url);
    if (parsed.resource === "github.com" && parsed.full_name) {
      return { github: `https://github.com/${parsed.full_name}` };
    }
  }
  return { github: void 0 };
}

const UnpluginInfo = unplugin.createUnplugin((option) => {
  let now;
  const root = path__default.resolve(option?.root ?? process__default.cwd());
  const ModuleName = {
    BuildTime: `${option?.prefix ?? "~build"}/time`,
    BuildGit: `${option?.prefix ?? "~build"}/git`,
    BuildInfo: `${option?.prefix ?? "~build"}/info`,
    BuildCI: `${option?.prefix ?? "~build"}/ci`,
    BuildMeta: `${option?.prefix ?? "~build"}/meta`,
    BuildPackage: `${option?.prefix ?? "~build"}/package`
  };
  return {
    name: "unplugin-info",
    buildStart() {
      now = /* @__PURE__ */ new Date();
    },
    resolveId(id) {
      if (Object.values(ModuleName).includes(id)) {
        return `\0${id}`;
      }
    },
    loadInclude(id) {
      if (!id.startsWith("\0"))
        return false;
      id = id.slice(1);
      return Object.values(ModuleName).includes(id);
    },
    async load(id) {
      if (!id.startsWith("\0"))
        return;
      id = id.slice(1);
      if (id === ModuleName.BuildTime) {
        return `const time = new Date(${now.getTime()})
export default time`;
      } else if (id === ModuleName.BuildInfo || id === ModuleName.BuildGit) {
        const info = await getRepoInfo(root, option?.git);
        if (info && option?.github) {
          info.github = option.github;
        }
        if (id === ModuleName.BuildInfo) {
          this.warn(
            `${ModuleName.BuildInfo} is deprecated, please migrate to ${ModuleName.BuildGit} and ${ModuleName.BuildCI}`
          );
        }
        if (!info) {
          this.warn("This may not be a git repo");
        }
        const keys = [
          .../* @__PURE__ */ new Set([
            "github",
            "sha",
            "abbreviatedSha",
            "branch",
            "tag",
            "tags",
            "lastTag",
            "author",
            "authorEmail",
            "authorDate",
            "committer",
            "committerEmail",
            "committerDate",
            "commitMessage",
            ...Object.keys(option?.git ?? {})
          ])
        ];
        const gen = (key) => {
          return `export const ${key} = ${info ? JSON.stringify(info[key]) : "null"}`;
        };
        return [
          id === ModuleName.BuildInfo ? `export const CI = ${ci__default.isCI ? `"${ci__default.name}"` : "null"}` : ``,
          ...keys.map((key) => gen(key))
        ].join("\n");
      } else if (id === ModuleName.BuildCI) {
        return [
          `export const isCI = ${ci__default.isCI !== null ? ci__default.isCI ? "true" : "false" : "null"}`,
          `export const isPR = ${ci__default.isPR !== null ? ci__default.isPR ? "true" : "false" : "null"}`,
          `export const name = ${ci__default.name !== null ? `\`${ci__default.name}\`` : "null"}`
        ].join("\n");
      } else if (id === ModuleName.BuildMeta) {
        const body = Object.entries(option?.meta ?? {}).map(
          ([key, value]) => `export const ${key} = ${JSON.stringify(value, null, 2)};`
        );
        return body.join("\n");
      } else if (id === ModuleName.BuildPackage) {
        const pkg = JSON.parse(fs__default.readFileSync(path__default.join(root, "package.json"), "utf-8"));
        const defaults = ["name", "version", "description", "keywords", "license", "author"];
        const keys = new Set(
          Array.isArray(option?.package) ? [...defaults, ...option.package] : typeof option?.package === "object" ? Object.entries(
            Object.fromEntries([
              ...defaults.map((d) => [d, true]),
              ...Object.entries(option.package)
            ])
          ).filter(([k, v]) => !!v).map(([k, v]) => k) : defaults
        );
        const resolved = {
          name: "",
          version: "0.0.0",
          description: "",
          keywords: [],
          license: "",
          author: "",
          ...pkg
        };
        const entries = [...keys].map((key) => [key, resolved[key]]);
        return entries.map(([key, value]) => `export const ${key} = ${JSON.stringify(value, null, 2)};`).join("\n");
      }
    },
    vite: {
      handleHotUpdate({ file, server }) {
        if (file === normalizePath(path__default.resolve(root, "package.json"))) {
          const module = server.moduleGraph.getModuleById("\0" + ModuleName.BuildPackage);
          if (module) {
            server.moduleGraph.invalidateModule(module);
            server.ws.send({
              type: "full-reload"
            });
          }
        }
      }
    }
  };
});
function normalizePath(filename) {
  return filename.split(path__default.win32.sep).join(path__default.posix.sep);
}

exports.UnpluginInfo = UnpluginInfo;
